﻿using Microsoft.EntityFrameworkCore;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.SearchModels;
using SecuritySystemContracts.StoragesContracts;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDatabaseImplement.Models;

namespace SecuritySystemDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new SecuritySystemDatabase();
            return context.Orders.Include(x => x.Secure).Include(x => x.Client).FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new SecuritySystemDatabase();
            var orders = context.Orders
            .Include(x => x.Secure)
            .Include(x => x.Client)
            .Select(x => x.GetViewModel)
            .ToList();
            if (model.Id.HasValue)
            {
                orders = orders.Where(x => x.Id == model.Id.Value).ToList();
            }
            if (model.DateFrom.HasValue)
            {
                orders = orders.Where(x => x.DateCreate >= model.DateFrom.Value).ToList();
            }
            if (model.DateTo.HasValue)
            {
                orders = orders.Where(x => x.DateCreate <= model.DateTo.Value).ToList();
            }
            if (model.ClientId.HasValue)
            {
                orders = orders.Where(x => x.ClientId == model.ClientId.Value).ToList();
            }
            return orders;
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new SecuritySystemDatabase();
            return context.Orders.Include(x => x.Secure).Include(x => x.Client).Select(x => x.GetViewModel).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new SecuritySystemDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x => x.Secure).Include(x => x.Client).FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new SecuritySystemDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders.Include(x => x.Secure).Include(x => x.Client).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new SecuritySystemDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
